@@ -63,7 +63,7 @@ void ResultWidget::paintEvent(QPaintEvent* /* p_paintEvent */)
 	paint.setPen(pen);
 
 	// get the font height; the font height doesn't change while painting
-	QFontMetrics & fm = * new QFontMetrics(paint.fontMetrics());
+	QFontMetrics fm(paint.fontMetrics());
 
 	// now we can correctly set the height of the widget
 	setMinimumHeight(2 * fm.lineSpacing() + 10);
