@@ -14,7 +14,7 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  USA
 **/
 
@@ -229,7 +229,6 @@ void KigPart::setupActions()
   KIconLoader* l = instance()->iconLoader();
   QPixmap tmp;
 
-  tmp = l->loadIcon( "editdelete", KIcon::Toolbar);
   aDeleteObjects = new KAction(
       i18n("&Delete Objects"), "editdelete", Key_Delete, this,
       SLOT(deleteObjects()), actionCollection(), "delete_objects");
@@ -243,14 +242,13 @@ void KigPart::setupActions()
   aCancelConstruction->setEnabled(false);
 
   aShowHidden = new KAction(
-    i18n("Unhide &All"), 0, this, SLOT( showHidden() ),
+    i18n("U&nhide All"), 0, this, SLOT( showHidden() ),
     actionCollection(), "edit_unhide_all");
   aShowHidden->setToolTip(i18n("Show all hidden objects"));
   aShowHidden->setEnabled( true );
 
-  tmp = l->loadIcon("gear", KIcon::Toolbar);
   aNewMacro = new KAction(
-    i18n("&New Macro..."), tmp, 0, this, SLOT(newMacro()),
+    i18n("&New Macro..."), "gear", 0, this, SLOT(newMacro()),
     actionCollection(), "macro_action");
   aNewMacro->setToolTip(i18n("Define a new macro"));
 
@@ -275,7 +273,7 @@ void KigPart::setupActions()
   a = KStdAction::fitToPage( m_widget, SLOT( slotRecenterScreen() ),
                              actionCollection() );
   // grr.. why isn't there an icon for this..
-  a->setIconSet( QIconSet( l->loadIcon( "view_fit_to_page", KIcon::User ) ) );
+  a->setIconSet( QIconSet( l->loadIcon( "view_fit_to_page", KIcon::Toolbar ) ) );
   a->setToolTip( i18n( "Recenter the screen on the document" ) );
   a->setWhatsThis( i18n( "Recenter the screen on the document" ) );
 
@@ -297,16 +295,14 @@ void KigPart::setupActions()
   a->setWhatsThis( i18n( "View this document full-screen." ) );
 
   // TODO: an icon for this..
-  tmp = l->loadIcon( "viewmagfit", KIcon::Toolbar );
   a = new KAction(
-    i18n( "&Select Shown Area" ), tmp, 0, m_widget, SLOT( zoomRect() ),
+    i18n( "&Select Shown Area" ), "viewmagfit", 0, m_widget, SLOT( zoomRect() ),
     actionCollection(), "view_select_shown_rect" );
   a->setToolTip( i18n( "Select the area that you want to be shown in the window." ) );
   a->setWhatsThis( i18n( "Select the area that you want to be shown in the window." ) );
 
-  tmp = l->loadIcon( "viewmag", KIcon::Toolbar );
   a = new KAction(
-    i18n( "S&elect Zoom Area" ), tmp, 0, m_widget, SLOT( zoomArea() ),
+    i18n( "S&elect Zoom Area" ), "viewmag", 0, m_widget, SLOT( zoomArea() ),
     actionCollection(), "view_zoom_area" );
 //  a->setToolTip( i18n( "Select the area that you want to be shown in the window." ) );
 //  a->setWhatsThis( i18n( "Select the area that you want to be shown in the window." ) );
@@ -394,16 +390,21 @@ bool KigPart::openFile()
         i18n( "You tried to open a document of type \"%1\"; unfortunately, "
               "Kig does not support this format. If you think the format in "
               "question would be worth implementing support for, you can "
-              "always ask me nicely on mailto:devriese@kde.org "
+              "always ask us nicely on mailto:toscano.pino@tiscali.it "
               "or do the work yourself and send me a patch."
           ).arg(mimeType->name()),
-        i18n( "Format not Supported" )
+        i18n( "Format Not Supported" )
         );
     return false;
   };
 
   KigDocument* newdoc = filter->load (m_file);
-  if ( !newdoc ) return false;
+  if ( !newdoc )
+  {
+    closeURL();
+    m_url = KURL();
+    return false;
+  }
   delete mdocument;
   mdocument = newdoc;
   coordSystemChanged( mdocument->coordinateSystem().id() );
@@ -426,7 +427,7 @@ bool KigPart::openFile()
 
 bool KigPart::saveFile()
 {
-  if ( m_file.isEmpty() ) return internalSaveAs();
+  if ( m_file.isEmpty() || m_bTemp ) return internalSaveAs();
   // mimetype:
   KMimeType::Ptr mimeType = KMimeType::findByPath ( m_file );
   if ( mimeType->name() != "application/x-kig" )
@@ -435,7 +436,7 @@ bool KigPart::saveFile()
     if( KMessageBox::warningYesNo( widget(),
                      i18n( "Kig does not support saving to any other file format than "
                            "its own. Save to Kig's format instead?" ),
-                     i18n( "Format not Supported" ) ) == KMessageBox::No )
+                     i18n( "Format Not Supported" ), i18n("Save Kig Format"), KStdGuiItem::cancel() ) == KMessageBox::No )
       return false;
     internalSaveAs();
   };
@@ -618,10 +619,10 @@ bool KigPart::internalSaveAs()
   if (file_name.isEmpty()) return false;
   else if ( QFileInfo( file_name ).exists() )
   {
-    int ret = KMessageBox::warningYesNo( m_widget,
+    int ret = KMessageBox::warningContinueCancel( m_widget,
                                          i18n( "The file \"%1\" already exists. Do you wish to overwrite it?" )
-                                         .arg( file_name ), i18n( "Overwrite File?" ) );
-    if ( ret != KMessageBox::Yes )
+                                         .arg( file_name ), i18n( "Overwrite File?" ), i18n("Overwrite") );
+    if ( ret != KMessageBox::Continue )
     {
       return false;
     }
