@@ -14,7 +14,7 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  USA
 **/
 
@@ -30,7 +30,7 @@
 
 #include "dict.h"
 
-#include <iostream.h>
+#include <iostream>
 #include <cassert>
 #include <sys/mman.h> 
 #include <stdio.h>
@@ -50,13 +50,18 @@ using namespace Dict;
 
 TextType Dict::textType(const QString &text)
 {
-	unsigned char first = QTextCodec::codecForName("eucJP")->fromUnicode(text)[0];
+	ushort first = text.at(0).unicode();
 	
-	if (first <= 128)
+	if (first < 0x3000)
 		return Text_Latin;
-	else if (first < 0xa8)
+	// else if (first < 0x3040) // CJK Symbols and Punctuation
+			// return Text_Kana;
+		// else if (first < 0x30A0) // Hiragana
+			// return Text_Kana;
+	else if (first < 0x3100) // Katakana
 		return Text_Kana;
-	else
+	
+	else /*if (first >= 0x3400 && first < 0x4DC0)*/ // CJK Unified Ideographs Extension A
 		return Text_Kanji;
 }
 
@@ -64,7 +69,7 @@ File::File(QString path, QString n)
 	: myName(n)
 	, dictFile(path)
 	, dictPtr((const unsigned char *)MAP_FAILED)
-	, indexFile(KGlobal::dirs()->saveLocation("appdata", "xjdx/", true) + QFileInfo(path).baseName() + ".xjdx")
+	, indexFile(KGlobal::dirs()->saveLocation("data", "kiten/xjdx/", true) + QFileInfo(path).baseName() + ".xjdx")
 	, indexPtr((const uint32_t *)MAP_FAILED)
 	, valid(false)
 {
@@ -248,7 +253,7 @@ void Index::loadDictList(QPtrList<File> &fileList, const QStringList &dictList,
 	}
 }
 
-QStringList Index::doSearch(File &file, QString text)
+QStringList Index::doSearch(File &file, const QString &text)
 {
 	// Do a binary search to find an entry that matches text
 	QTextCodec &codec = *QTextCodec::codecForName("eucJP");
@@ -273,7 +278,7 @@ QStringList Index::doSearch(File &file, QString text)
 		else if (comp > 0)
 			lo = cur + 1;
 	}
-	while(hi >= lo && comp != 0);
+	while(hi >= lo && comp != 0 && !(hi == 0 && lo == 0));
 	QStringList results;
 	// A match?
 	if (comp == 0)
@@ -355,7 +360,7 @@ SearchResult Index::scanResults(QRegExp regexp, QStringList results, bool common
 	return ret;
 }
 
-SearchResult Index::search(QRegExp regexp, QString text, bool common)
+SearchResult Index::search(QRegExp regexp, const QString &text, bool common)
 {
 	QStringList results;
 	for (QPtrListIterator<File> file(dictFiles); *file; ++file)
@@ -407,7 +412,7 @@ SearchResult Index::scanKanjiResults(QRegExp regexp, QStringList results, bool c
 	return ret;
 }
 
-SearchResult Index::searchKanji(QRegExp regexp, QString text,  bool common)
+SearchResult Index::searchKanji(QRegExp regexp, const QString &text,  bool common)
 {
 	QStringList results;
 	for (QPtrListIterator<File> file(kanjiDictFiles); *file; ++file)
@@ -422,7 +427,7 @@ SearchResult Index::searchKanji(QRegExp regexp, QString text,  bool common)
 	return res;
 }
 
-SearchResult Index::searchPrevious(QRegExp regexp, QString text, SearchResult list, bool common)
+SearchResult Index::searchPrevious(QRegExp regexp, const QString &text, SearchResult list, bool common)
 {
 	SearchResult res;
 
@@ -435,6 +440,56 @@ SearchResult Index::searchPrevious(QRegExp regexp, QString text, SearchResult li
 	return res;
 }
 
+QRegExp Dict::Index::createRegExp(SearchType type, const QString &text, DictionaryType dictionaryType, bool caseSensitive)
+{
+	QString regExp;
+	switch (type)
+	{
+	case Search_Beginning:
+		switch (textType(text))
+		{
+		case Dict::Text_Latin:
+			regExp = "\\W%1";
+			break;
+
+		case Dict::Text_Kana:
+			if (dictionaryType == Kanjidict)
+				regExp = "\\W%1";
+			else // edict
+				regExp = "\\[%1";
+			break;
+
+		case Dict::Text_Kanji:
+			regExp = "^%1";
+		}
+		break;
+	
+	case Search_FullWord:
+		switch (textType(text))
+		{
+		case Dict::Text_Latin:
+			regExp = "\\W%1\\W";
+			break;
+
+		case Dict::Text_Kana:
+			if (dictionaryType == Kanjidict)
+				regExp = " %1 ";
+			else // edict
+				regExp = "\\[%1\\]";
+			break;
+
+		case Dict::Text_Kanji:
+			regExp = "^%1\\W";
+		}
+		break;
+	
+	case Search_Anywhere:
+		regExp = "%1";
+	}
+
+	return QRegExp(regExp.arg(text), caseSensitive);
+}
+
 int Index::stringCompare(File &file, int index, QCString str)
 {
 	return eucStringCompare(file.lookup(index), str);
@@ -728,7 +783,7 @@ Entry::Entry(const QString & kanji, const QString & reading, const QStringList &
 {
 }
 
-Entry::Entry(QString &kanji, QStringList &readings, QStringList &meanings, unsigned int grade, unsigned int freq, unsigned int strokes, unsigned int miscount)
+Entry::Entry(const QString &kanji, QStringList &readings, QStringList &meanings, unsigned int grade, unsigned int freq, unsigned int strokes, unsigned int miscount)
 	: DictName(QString::fromLatin1("__NOTSET"))
 	, Header(QString::fromLatin1("__NOTSET"))
 	, Meanings(meanings)
