@@ -17,7 +17,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.     */
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.     */
 
 /* Changed: ignore all rc stuff. use args 1 and 2 for input/output file.
   -- jason */
@@ -148,7 +148,6 @@ unsigned char **argv;
   if(jindex == NULL)
   {
 	  fprintf(stderr,"malloc() for index table failed.\n");
-	  fclose(fp);
 	  exit(1);
   }
   printf("Parsing.... \n");
